@@ -19,10 +19,10 @@ This module contains the main classes and methods for Epylog.
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: __init__.py,v 1.43.2.6 2005/03/31 21:14:12 icon Exp $
+# $Id$
 #
 # @Author Konstantin Ryabitsev <icon@linux.duke.edu>
-# @version $Date: 2005/03/31 21:14:12 $
+# @version $Date$
 #
 
 import ConfigParser
@@ -47,11 +47,11 @@ from report import Report
 from module import Module
 from log import LogTracker
 
-VERSION = 'Epylog-1.0.3'
+VERSION = 'Epylog-1.0.7'
 CHUNK_SIZE = 8192
 GREP_LINES = 10000
 QUEUE_LIMIT = 500
-LOG_SPLIT_RE = re.compile(r'(.{15,15}) (\S+) (.*)$')
+LOG_SPLIT_RE = re.compile(r'(.{15,15})\s+(\S+)\s+(.*)$')
 SYSLOG_NG_STRIP = re.compile(r'.*[@/]')
 MESSAGE_REPEATED_RE = re.compile(r'last message repeated (\S+) times')
 
@@ -125,6 +125,15 @@ class NoSuchLogError(exceptions.Exception):
         logger.put(5, '!NoSuchLogError: %s' % message)
         self.args = message
 
+class EmptyLogError(exceptions.Exception):
+    """
+    This exception is raised when Epylog finds an empty logfile.
+    """
+    def __init__(self, message, logger):
+        exceptions.Exception.__init__(self)
+        logger.put(5, '!EmptyLogError: %s' % message)
+        self.args = message
+
 class GenericError(exceptions.Exception):
     """
     This exception is raised for all other Epylog conditions.
