@@ -21,10 +21,10 @@ __init__ and publish methods.
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 # 02111-1307, USA.
 #
-# $Id: publishers.py,v 1.19.2.1 2004/02/09 20:31:42 icon Exp $
+# $Id$
 #
 # @Author Konstantin Ryabitsev <icon@linux.duke.edu>
-# @version $Date: 2004/02/09 20:31:42 $
+# @version $Date$
 #
 
 import epylog
@@ -202,6 +202,40 @@ class MailPublisher:
         logger.put(5, 'format=%s' % self.format)
         logger.put(5, 'rawlogs=%d' % self.rawlogs)
         logger.put(5, 'smtpserv=%s' % self.smtpserv)
+
+        try:
+            self.gpg_encrypt = config.getboolean(self.section, 'gpg_encrypt')
+
+            try:
+                self.gpg_keyringdir = config.get(self.section, 'gpg_keyringdir')
+            except:
+                self.gpg_keyringdir = None
+
+            try:
+                gpg_recipients = config.get(self.section, 'gpg_recipients')
+                keyids = gpg_recipients.split(',')
+                self.gpg_recipients = []
+                for keyid in keyids:
+                    keyid = keyid.strip()
+                    logger.put(5, 'adding gpg_recipient=%s' % keyid)
+                    self.gpg_recipients.append(keyid)
+            except:
+                # Will use all recipients found in the keyring
+                self.gpg_recipients = None
+
+            try:
+                gpg_signers = config.get(self.section, 'gpg_signers')
+                keyids = gpg_signers.split(',')
+                self.gpg_signers = []
+                for keyid in keyids:
+                    keyid = keyid.strip()
+                    logger.put(5, 'adding gpg_signer=%s' % keyid)
+                    self.gpg_signers.append(keyid)
+            except:
+                self.gpg_signers = None
+
+        except:
+            self.gpg_encrypt = 0
         
         logger.put(5, '<MailPublisher.__init__')
         
@@ -260,48 +294,149 @@ class MailPublisher:
                 self.gzlogs = outfh.read()
             outfh.close()
             
-        ##
-        # Using MimeWriter, since package 'email' doesn't come with rhl-7.3
-        # Suck-o.
-        #
         logger.puthang(3, 'Creating an email message')
-        import StringIO, MimeWriter
-        fh = StringIO.StringIO()
+        from email.mime.base      import MIMEBase
+        from email.mime.text      import MIMEText
+        from email.mime.multipart import MIMEMultipart
+
         logger.put(5, 'Creating a main header')
-        mw = MimeWriter.MimeWriter(fh)
-        mw.addheader('Subject', title)
-        if len(self.mailto) > 1:
-            import string
-            tostr = string.join(self.mailto, ', ')
-        else:
-            tostr = self.mailto[0]
-        mw.addheader('To', tostr)
-        mw.addheader('X-Mailer', epylog.VERSION)
-        self.mw = mw
+
+        root_part = MIMEMultipart('mixed')
+        root_part.preamble = 'This is a multi-part message in MIME format.'
+
+        logger.put(5, 'Creating the text/plain part')
+        text_part = MIMEText(self.plainrep, 'plain', 'utf-8')
+        logger.put(5, 'Creating the text/html part')
+        html_part = MIMEText(self.htmlrep, 'html', 'utf-8')
         
-        if self.rawlogs > 0 and self.format == 'both':
-            logger.put(5, 'Making a html + plain + gzip message')
-            self._mk_both_rawlogs()
-        elif self.rawlogs > 0 and self.format == 'html':
-            logger.put(5, 'Making a html + gzip message')
-            self._mk_html_rawlogs()
-        elif self.rawlogs > 0 and self.format == 'plain':
-            logger.put(5, 'Making a plain + gzip message')
-            self._mk_plain_rawlogs()
-        elif self.rawlogs == 0 and self.format == 'both':
-            logger.put(5, 'Making a html + plain message')
-            self._mk_both_nologs()
-        elif self.rawlogs == 0 and self.format == 'html':
-            logger.put(5, 'Making a html message')
-            self._mk_html_nologs()
-        elif self.rawlogs == 0 and self.format == 'plain':
-            logger.put(5, 'Making a plain message')
-            self._mk_plain_nologs()
+        if self.rawlogs > 0:
+            logger.put(5, 'Creating the application/x-gzip part')
+            attach_part = MIMEBase('application', 'x-gzip')
+            attach_part.set_payload(self.gzlogs)
+            from email.encoders import encode_base64
+            logger.put(5, 'Encoding the gzipped raw logs with base64')
+            encode_base64(attach_part)
+            attach_part.add_header('Content-Disposition', 'attachment', 
+                                   filename='raw.log.gz')
+        
+        if self.format == 'both':
+            # create another multipart for text+html
+            alt_part = MIMEMultipart('alternative')
+            alt_part.attach(text_part)
+            alt_part.attach(html_part)
+            root_part.attach(alt_part)
+        elif self.format == 'html':
+            root_part.attach(html_part)
+        elif self.format == 'plain':
+            root_part.attach(text_part)
+
+        if self.rawlogs > 0:
+            root_part.attach(attach_part)
+
         logger.endhang(3)
 
-        fh.seek(0)
-        msg = fh.read()
-        fh.close()
+        if self.gpg_encrypt:
+            logger.puthang(3, 'Encrypting the message')
+
+            from StringIO import StringIO
+            try:
+                import gpgme
+
+                if self.gpg_keyringdir and os.path.exists(self.gpg_keyringdir):
+                    logger.put(5, 'Setting keyring dir to %s' % self.gpg_keyringdir)
+                    os.environ['GNUPGHOME'] = self.gpg_keyringdir
+
+                msg = root_part.as_string()
+                logger.put(5, 'Cleartext follows')
+                logger.put(5, msg)
+                logger.put(5, 'Cleartext ends')
+
+                cleartext = StringIO(msg)
+                ciphertext = StringIO()
+
+                ctx = gpgme.Context()
+                ctx.armor = True
+
+                recipients = []
+                signers = []
+                logger.put(5, 'self.gpg_recipients = %s' % self.gpg_recipients)
+                logger.put(5, 'self.gpg_signers = %s' % self.gpg_signers)
+
+                if self.gpg_recipients is not None:
+                    for recipient in self.gpg_recipients:
+                        logger.puthang(5, 'Looking for an encryption key for %s' % recipient)
+                        recipients.append(ctx.get_key(recipient))
+                        logger.endhang(5)
+                else:
+                    for key in ctx.keylist():
+                        for subkey in key.subkeys:
+                            if subkey.can_encrypt:
+                                logger.put(5, 'Found can_encrypt key=%s' % subkey.keyid)
+                                recipients.append(key)
+                                break
+
+                if self.gpg_signers is not None:
+                    for signer in self.gpg_signers:
+                        logger.puthang(5, 'Looking for a signing key for %s' % signer)
+                        signers.append(ctx.get_key(signer))
+                        logger.endhang(5)
+
+                if len(signers) > 0:
+                    logger.puthang(3, 'Encrypting and signing the report')
+                    ctx.signers = signers
+                    ctx.encrypt_sign(recipients, gpgme.ENCRYPT_ALWAYS_TRUST,
+                                     cleartext, ciphertext)
+                    logger.endhang(3)
+
+                else:
+                    logger.puthang(3, 'Encrypting the report')
+                    ctx.encrypt(recipients, gpgme.ENCRYPT_ALWAYS_TRUST,
+                                cleartext, ciphertext)
+                    logger.endhang(3)
+
+                logger.puthang(5, 'Creating the MIME envelope for PGP')
+
+                gpg_envelope_part = MIMEMultipart('encrypted')
+                gpg_envelope_part.set_param('protocol', 'application/pgp-encrypted', 
+                                            header='Content-Type')
+                gpg_envelope_part.preamble = 'This is an OpenPGP/MIME encrypted message (RFC 2440 and 3156)'
+
+                gpg_mime_version_part = MIMEBase('application', 'pgp-encrypted')
+                gpg_mime_version_part.add_header('Content-Disposition', 
+                                                 'PGP/MIME version identification')
+                gpg_mime_version_part.set_payload('Version: 1')
+
+                gpg_payload_part = MIMEBase('application', 'octet-stream', 
+                                            name='encrypted.asc')
+                gpg_payload_part.add_header('Content-Disposition', 
+                                            'OpenPGP encrypted message')
+                gpg_payload_part.add_header('Content-Disposition', 'inline',
+                                            filename='encrypted.asc')
+                gpg_payload_part.set_payload(ciphertext.getvalue())
+
+                gpg_envelope_part.attach(gpg_mime_version_part)
+                gpg_envelope_part.attach(gpg_payload_part)
+
+                # envelope becomes the new root part
+                root_part = gpg_envelope_part
+
+                logger.endhang(5)
+
+            except ImportError:
+                logger.endhang(3)
+                logger.put(0, 'Install pygpgme for GPG encryption support.')
+                logger.put(0, 'Not mailing the report out of caution.')
+                return
+
+            logger.endhang(3)
+
+        root_part['Subject'] = title
+        root_part['To'] = ', '.join(self.mailto)
+        root_part['X-Mailer'] = epylog.VERSION
+        
+        logger.put(5, 'Creating the message as string')
+        msg = root_part.as_string()
+
         logger.put(5, 'Message follows')
         logger.put(5, msg)
         logger.put(5, 'End of message')
@@ -312,163 +447,9 @@ class MailPublisher:
         else:
             fromaddr = 'root@%s' % socket.gethostname()
             mail_smtp(self.smtpserv, fromaddr, self.mailto, msg, logger)
-        logger.put(1, 'Mailed the report to: %s' % tostr)
+        logger.put(1, 'Mailed the report to: %s' % ','.join(self.mailto))
         logger.put(5, '<MailPublisher.publish')
 
-    def _mk_both_rawlogs(self):
-        """
-        Make an email message that includes html, plaintext, and gzipped raw
-        logs sections. Most painful.
-        """
-        self.logger.put(5, '>MailPublisher._mk_both_rawlogs')
-        import base64
-        logger = self.logger
-        mixed_mw = self.mw
-        mixed_mw.addheader('Mime-Version', '1.0')
-        logger.put(5, 'Creating a multipart/mixed part')
-        mixed_mw.startmultipartbody('mixed')
-
-        logger.put(5, 'Creating a multipart/alternative part')
-        alt_mw = mixed_mw.nextpart()
-        alt_mw.startmultipartbody('alternative')
-
-        logger.put(5, 'Creating a text/plain part')
-        plain_mw = alt_mw.nextpart()
-        plain_mw.addheader('Content-Transfer-Encoding', '8bit')
-        plain_fh = plain_mw.startbody('text/plain; charset=iso-8859-1')
-        plain_fh.write(self.plainrep)
-
-        logger.put(5, 'Creating a text/html part')
-        html_mw = alt_mw.nextpart()
-        html_mw.addheader('Content-Transfer-Encoding', '8bit')
-        html_fh = html_mw.startbody('text/html; charset=iso-8859-1')
-        html_fh.write(self.htmlrep)
-
-        alt_mw.lastpart()
-        logger.put(5, 'Creating an application/gzip part')
-        gzip_mw = mixed_mw.nextpart()
-        gzip_mw.addheader('Content-Transfer-Encoding', 'base64')
-        gzip_mw.addheader('Content-Disposition',
-                          'attachment; filename=rawlogs.gz')
-        gzip_fh = gzip_mw.startbody('application/gzip; NAME=rawlogs.gz')
-        gzip_fh.write(base64.encodestring(self.gzlogs))
-        mixed_mw.lastpart()
-        self.logger.put(5, '<MailPublisher._mk_both_rawlogs')
-
-    def _mk_html_rawlogs(self):
-        """
-        Make an email message that includes html and gzipped raw logs sections.
-        """
-        self.logger.put(5, '>MailPublisher._mk_html_rawlogs')
-        import base64
-        logger = self.logger
-        mixed_mw = self.mw
-        mixed_mw.addheader('Mime-Version', '1.0')
-        logger.put(5, 'Creating a multipart/mixed part')
-        mixed_mw.startmultipartbody('mixed')
-
-        logger.put(5, 'Creating a text/html part')
-        html_mw = mixed_mw.nextpart()
-        html_mw.addheader('Content-Transfer-Encoding', '8bit')
-        html_fh = html_mw.startbody('text/html; charset=iso-8859-1')
-        html_fh.write(self.htmlrep)
-
-        logger.put(5, 'Creating an application/gzip part')
-        gzip_mw = mixed_mw.nextpart()
-        gzip_mw.addheader('Content-Transfer-Encoding', 'base64')
-        gzip_mw.addheader('Content-Disposition',
-                          'attachment; filename=rawlogs.gz')
-        gzip_fh = gzip_mw.startbody('application/gzip; NAME=rawlogs.gz')
-        gzip_fh.write(base64.encodestring(self.gzlogs))
-        mixed_mw.lastpart()
-        self.logger.put(5, '<MailPublisher._mk_html_rawlogs')
-
-    def _mk_plain_rawlogs(self):
-        """
-        Make an email message that includes plaintext and gzipped raw logs
-        sections.
-        """
-        self.logger.put(5, '>MailPublisher._mk_plain_rawlogs')
-        import base64
-        logger = self.logger
-        mixed_mw = self.mw
-        mixed_mw.addheader('Mime-Version', '1.0')
-        logger.put(5, 'Creating a multipart/mixed part')
-        mixed_mw.startmultipartbody('mixed')
-
-        logger.put(5, 'Creating a text/plain part')
-        plain_mw = mixed_mw.nextpart()
-        plain_mw.addheader('Content-Transfer-Encoding', '8bit')
-        plain_fh = plain_mw.startbody('text/plain; charset=iso-8859-1')
-        plain_fh.write(self.plainrep)
-
-        logger.put(5, 'Creating an application/gzip part')
-        gzip_mw = mixed_mw.nextpart()
-        gzip_mw.addheader('Content-Transfer-Encoding', 'base64')
-        gzip_mw.addheader('Content-Disposition',
-                          'attachment; filename=rawlogs.gz')
-        gzip_fh = gzip_mw.startbody('application/gzip; NAME=rawlogs.gz')
-        gzip_fh.write(base64.encodestring(self.gzlogs))
-        mixed_mw.lastpart()
-        self.logger.put(5, '<MailPublisher._mk_plain_rawlogs')
-
-    def _mk_both_nologs(self):
-        """
-        Make a message that just includes html and plaintext sections.
-        """
-        self.logger.put(5, '>MailPublisher._mk_both_nologs')
-        logger = self.logger
-        alt_mw = self.mw
-        alt_mw.addheader('Mime-Version', '1.0')
-        logger.put(5, 'Creating a multipart/alternative part')
-        alt_mw.startmultipartbody('alternative')
-
-        logger.put(5, 'Creating a text/plain part')
-        plain_mw = alt_mw.nextpart()
-        plain_mw.addheader('Content-Transfer-Encoding', '8bit')
-        plain_fh = plain_mw.startbody('text/plain; charset=iso-8859-1')
-        plain_fh.write(self.plainrep)
-
-        logger.put(5, 'Creating a text/html part')
-        html_mw = alt_mw.nextpart()
-        html_mw.addheader('Content-Transfer-Encoding', '8bit')
-        html_fh = html_mw.startbody('text/html; charset=iso-8859-1')
-        html_fh.write(self.htmlrep)
-
-        alt_mw.lastpart()
-        self.logger.put(5, '<MailPublisher._mk_both_nologs')
-
-    def _mk_html_nologs(self):
-        """
-        Make a message that just includes HTML-formatted section.
-        """
-        self.logger.put(5, '>MailPublisher._mk_html_nologs')
-        logger = self.logger
-        alt_mw = self.mw
-        alt_mw.addheader('Mime-Version', '1.0')
-        logger.put(5, 'Creating a multipart/alternative part')
-        alt_mw.startmultipartbody('alternative')
-        logger.put(5, 'Creating a text/html part')
-        html_mw = alt_mw.nextpart()
-        html_mw.addheader('Content-Transfer-Encoding', '8bit')
-        html_fh = html_mw.startbody('text/html; charset=iso-8859-1')
-        html_fh.write(self.htmlrep)
-        alt_mw.lastpart()
-        self.logger.put(5, '<MailPublisher._mk_html_nologs')
-
-    def _mk_plain_nologs(self):
-        """
-        Make a message that just includes a plaintext-formatted section.
-        """
-        self.logger.put(5, '>MailPublisher._mk_plain_nologs')
-        logger = self.logger
-        plain_mw = self.mw
-        logger.put(5, 'Creating a text/plain part')
-        plain_mw.addheader('Content-Transfer-Encoding', '8bit')
-        plain_fh = plain_mw.startbody('text/plain; charset=iso-8859-1')
-        plain_fh.write(self.plainrep)
-        self.logger.put(5, '<MailPublisher._mk_plain_nologs')
-
 
 class FilePublisher:
     """
@@ -501,6 +482,12 @@ class FilePublisher:
         self._prune_old(path, dirmask, expire)
         self.path = os.path.join(path, self.dirname)
 
+        logger.put(3, 'Looking if we should save rawlogs')
+        try: self.save_rawlogs = config.getboolean(sec, 'save_rawlogs')
+        except: self.save_rawlogs = 0
+        if self.save_rawlogs:
+            logger.put(3, 'Saving raw logs in the reports directory')
+        
         logger.put(3, 'Checking if notify is set')
         self.notify = []
         try:
@@ -586,34 +573,31 @@ class FilePublisher:
         if self.notify:
             logger.puthang(3, 'Creating an email message')
             publoc = '%s/%s/%s' % (self.pubroot, self.dirname, filename)
-            msg = 'New Epylog report is available at:\r\n%s' % publoc
-            import StringIO, MimeWriter
-            fh = StringIO.StringIO()
-            logger.put(3, 'Creating a main header')
-            mw = MimeWriter.MimeWriter(fh)
-            mw.addheader('Subject', '%s (report notification)' % title)
-            tostr = ', '.join(self.notify)
-            mw.addheader('To', tostr)
-            mw.addheader('X-Mailer', epylog.VERSION)
-            mw.addheader('Content-Transfer-Encoding', '8bit')
-            bfh = mw.startbody('text/plain; charset=iso-8859-1')
-            bfh.write(msg)
-            fh.seek(0)
-            msg = fh.read()
-            fh.close()
+
+            from email.mime.text import MIMEText
+            eml = MIMEText('New Epylog report is available at:\r\n%s' % publoc)
+
+            eml['Subject'] = '%s (report notification)' % title
+            eml['To'] = ', '.join(self.notify)
+            eml['X-Mailer'] = epylog.VERSION
+
+            msg = eml.as_string()
+
             logger.put(3, 'Figuring out if we are using sendmail or smtplib')
             if re.compile('^/').search(self.smtpserv):
                 mail_sendmail(self.smtpserv, msg, logger)
             else:
                 fromaddr = 'root@%s' % socket.gethostname()
                 mail_smtp(self.smtpserv, fromaddr, self.notify, msg, logger)
-            logger.put(1, 'Notification mailed to: %s' % tostr)
+            logger.put(1, 'Notification mailed to: %s' % ','.join(self.notify))
 
-        logfilen = '%s.log' % self.filename
-        logfile = os.path.join(self.path, '%s.gz' % logfilen)
-        logger.put(3, 'Gzipping logs and writing them to %s' % logfilen)
-        outfh = open(logfile, 'w+')
-        do_chunked_gzip(rawfh, outfh, logfilen, logger)
-        outfh.close()
-        logger.put(1, 'Gzipped logs saved in: %s' % self.path)
+        if self.save_rawlogs:
+            logfilen = '%s.log' % self.filename
+            logfile = os.path.join(self.path, '%s.gz' % logfilen)
+            logger.put(3, 'Gzipping logs and writing them to %s' % logfilen)
+            outfh = open(logfile, 'w+')
+            do_chunked_gzip(rawfh, outfh, logfilen, logger)
+            outfh.close()
+            logger.put(1, 'Gzipped logs saved in: %s' % self.path)
+        
         logger.put(5, '<FilePublisher.publish')
